@@ -8,12 +8,32 @@ import uuid
 from pathlib import Path
 import click
 from typing import Optional
+from importlib.metadata import version as get_version
 
 from .config import Config
 from .client import RelayClient
 from .hooks import HooksInstaller
 
 
+def check_for_updates() -> Optional[str]:
+    """Check PyPI for newer version. Returns latest version if update available, None otherwise."""
+    try:
+        import httpx
+        current = get_version("watchcode-cli")
+        response = httpx.get("https://pypi.org/pypi/watchcode-cli/json", timeout=2.0)
+        if response.status_code == 200:
+            latest = response.json()["info"]["version"]
+            # Simple version comparison (works for x.y.z format)
+            if latest != current:
+                current_parts = [int(x) for x in current.split(".")]
+                latest_parts = [int(x) for x in latest.split(".")]
+                if latest_parts > current_parts:
+                    return latest
+    except Exception:
+        pass  # Don't fail on update check errors
+    return None
+
+
 def install_scripts() -> dict:
     """Install hook scripts to ~/.watchcode/ directory.
 
@@ -135,11 +155,29 @@ def poll_for_response(auth_token: str, notification_id: str, relay_url: str, tim
     return None
 
 
-@click.group()
+@click.group(invoke_without_command=True)
 @click.version_option(package_name="watchcode-cli")
-def main():
+@click.pass_context
+def main(ctx):
     """WatchCode CLI - Send Claude Code notifications to your Apple Watch."""
-    pass
+    # Check for updates (non-blocking, fails silently)
+    latest = check_for_updates()
+    if latest:
+        current = get_version("watchcode-cli")
+        click.echo(click.style(f"Update available: {current} → {latest}", fg="yellow"))
+        click.echo(click.style("Run: pipx upgrade watchcode-cli", fg="yellow"))
+        click.echo()
+
+    # If no command given, show help + setup hint if not configured
+    if ctx.invoked_subcommand is None:
+        click.echo(ctx.get_help())
+        click.echo()
+
+        config = Config()
+        if not config.is_configured():
+            click.echo(click.style("Getting started:", fg="green", bold=True))
+            click.echo("  Run: watchcode setup")
+            click.echo()
 
 
 def _setup_manual(config: Config) -> str:
