@@ -12,10 +12,9 @@ from keyring.errors import KeyringError
 class Config:
     """Manages WatchCode configuration files."""
 
-    DEFAULT_RELAY_URL = "https://watchcode-relay.sydneyworldbank.workers.dev"
+    DEFAULT_RELAY_URL = "https://relay.vgbndg.net"
     CONFIG_DIR = Path.home() / ".watchcode"
     CONFIG_FILE = CONFIG_DIR / "config.json"
-    QUEUE_FILE = CONFIG_DIR / "offline_queue.json"
 
     # Keychain service name for secure token storage
     KEYCHAIN_SERVICE = "com.watchcode.cli"
@@ -25,7 +24,6 @@ class Config:
         """Initialize config manager."""
         self.config_dir = self.CONFIG_DIR
         self.config_file = self.CONFIG_FILE
-        self.queue_file = self.QUEUE_FILE
         self._keychain_available = None
 
     def ensure_config_dir(self) -> None:
@@ -242,45 +240,6 @@ class Config:
 
         return "not configured"
 
-    def load_queue(self) -> list:
-        """Load offline notification queue.
-
-        Returns:
-            List of queued notifications.
-        """
-        if not self.queue_file.exists():
-            return []
-
-        try:
-            with open(self.queue_file, 'r') as f:
-                return json.load(f)
-        except (json.JSONDecodeError, IOError):
-            return []
-
-    def save_queue(self, queue: list) -> None:
-        """Save offline notification queue.
-
-        Args:
-            queue: List of notifications to queue.
-        """
-        self.ensure_config_dir()
-        with open(self.queue_file, 'w') as f:
-            json.dump(queue, f, indent=2)
-
-    def add_to_queue(self, notification: Dict[str, Any]) -> None:
-        """Add a notification to the offline queue.
-
-        Args:
-            notification: Notification data to queue.
-        """
-        queue = self.load_queue()
-        queue.append(notification)
-        self.save_queue(queue)
-
-    def clear_queue(self) -> None:
-        """Clear the offline queue."""
-        self.save_queue([])
-
     def format_token_display(self, token: str) -> str:
         """Format token for display with dashes (XXXX-XXXX-XXXX).
 
