@@ -136,6 +136,157 @@ def press_enter() -> bool:
         return False
 
 
+# ============================================================================
+# VOICE INPUT NAVIGATION FUNCTIONS
+# Maps voice text to option numbers and navigates using arrow keys
+# ============================================================================
+
+def match_voice_to_option(voice_text: str, options: list) -> int:
+    """Return 1-indexed option number that best matches voice input.
+
+    Handles various voice input patterns:
+    - Direct numbers: "1", "2", "3"
+    - Number words: "one", "two", "first", "second"
+    - Option labels: "Allow", "Deny", "Hello World"
+    - Partial matches: "hello" matches "Hello World"
+
+    Args:
+        voice_text: The text spoken by user (e.g., "Allow", "two", "Hello")
+        options: List of option dicts with 'label' keys
+
+    Returns:
+        1-indexed option number (1 if no match found)
+    """
+    voice_lower = voice_text.lower().strip()
+
+    # Direct number
+    if voice_lower.isdigit():
+        num = int(voice_lower)
+        if 1 <= num <= len(options):
+            return num
+        return 1  # Out of range, default to 1
+
+    # Number words
+    number_words = {
+        "one": 1, "two": 2, "three": 3, "four": 4, "five": 5,
+        "six": 6, "seven": 7, "eight": 8, "nine": 9, "ten": 10,
+        "first": 1, "second": 2, "third": 3, "fourth": 4, "fifth": 5,
+        "1st": 1, "2nd": 2, "3rd": 3, "4th": 4, "5th": 5
+    }
+    if voice_lower in number_words:
+        num = number_words[voice_lower]
+        if 1 <= num <= len(options):
+            return num
+        return 1
+
+    # Extract option labels
+    option_labels = [opt.get('label', '') for opt in options]
+
+    # Exact match (case-insensitive)
+    for i, label in enumerate(option_labels, 1):
+        if voice_lower == label.lower():
+            return i
+
+    # Voice contains label (e.g., "option allow" matches "Allow")
+    for i, label in enumerate(option_labels, 1):
+        if label.lower() in voice_lower:
+            return i
+
+    # Label contains voice (e.g., "hello" matches "Hello World")
+    for i, label in enumerate(option_labels, 1):
+        if voice_lower in label.lower():
+            return i
+
+    # No match - default to first option
+    log_debug(f"No match for voice '{voice_text}', defaulting to option 1")
+    return 1
+
+
+def navigate_to_option(option_num: int) -> bool:
+    """Navigate to option using arrow keys.
+
+    Claude Code's terminal prompt starts at option 1 (highlighted).
+    Press Down arrow (option_num - 1) times to reach option N.
+    Press Enter to select.
+
+    Uses osascript:
+    - key code 125 = Down arrow
+    - key code 36 = Enter/Return
+
+    Args:
+        option_num: 1-indexed option number to select
+
+    Returns:
+        True if navigation succeeded, False on error.
+    """
+    if not KEYBOARD_INJECTION_ENABLED:
+        log_debug("Keyboard injection disabled, cannot navigate")
+        return False
+
+    log_debug(f"Navigating to option {option_num} via arrow keys")
+
+    try:
+        # Press Down arrow (option_num - 1) times
+        for i in range(option_num - 1):
+            script = 'tell application "System Events" to key code 125'
+            result = subprocess.run(
+                ['osascript', '-e', script],
+                capture_output=True,
+                timeout=5
+            )
+            if result.returncode != 0:
+                log_debug(f"⚠️ Down arrow failed at step {i+1}: {result.stderr.decode()[:100]}")
+                return False
+            time.sleep(0.05)  # Small delay between keypresses
+
+        # Press Enter to select
+        script = 'tell application "System Events" to key code 36'
+        result = subprocess.run(
+            ['osascript', '-e', script],
+            capture_output=True,
+            timeout=5
+        )
+
+        if result.returncode == 0:
+            log_debug(f"✓ Selected option {option_num} via arrow navigation")
+            return True
+        else:
+            log_debug(f"⚠️ Enter key failed: {result.stderr.decode()[:100]}")
+            return False
+
+    except Exception as e:
+        log_debug(f"⚠️ Arrow navigation error: {e}")
+        return False
+
+
+def inject_voice_response(voice_text: str, options: list) -> bool:
+    """Inject voice response by navigating to matching option.
+
+    This is the main function for handling voice input from Watch.
+    Maps the voice text to an option number and uses arrow key navigation.
+
+    Args:
+        voice_text: The spoken text from Watch (e.g., "Allow", "two")
+        options: List of option dicts with 'label' keys
+
+    Returns:
+        True if injection succeeded, False on error.
+    """
+    if not voice_text or not options:
+        log_debug("⚠️ Missing voice_text or options for voice navigation")
+        return False
+
+    # Match voice to option number
+    option_num = match_voice_to_option(voice_text, options)
+    log_debug(f"Voice '{voice_text}' matched to option {option_num}")
+
+    # Small delay before navigation to ensure terminal is ready
+    time.sleep(0.1)
+
+    # Navigate using arrow keys
+    return navigate_to_option(option_num)
+
+
 def type_and_submit(text: str) -> bool:
     """Type text and press Enter to submit.
 
@@ -252,7 +403,7 @@ def launch_background_responder(
     log_debug(f"Launching background responder for notification {notification_id[:8]}...")
 
     try:
-        # Python script to run in background - polls relay and injects keystroke
+        # Python script to run in background - polls relay and uses arrow navigation
         python_script = f'''
 import urllib.request
 import json
@@ -265,6 +416,13 @@ OPTION_LABELS = {labels_json}
 RELAY_URL = "https://relay.vgbndg.net"
 TIMEOUT = {timeout}
 
+# Number words for voice input matching
+NUMBER_WORDS = {{
+    "one": 1, "two": 2, "three": 3, "four": 4, "five": 5,
+    "first": 1, "second": 2, "third": 3, "fourth": 4, "fifth": 5,
+    "1st": 1, "2nd": 2, "3rd": 3, "4th": 4, "5th": 5
+}}
+
 def poll_response():
     url = f"{{RELAY_URL}}/response/{{AUTH_TOKEN}}/{{NOTIFICATION_ID}}"
     try:
@@ -279,22 +437,47 @@ def poll_response():
         pass
     return None
 
-def inject(text):
-    # Target Terminal.app specifically, paste + enter
+def match_voice_to_option(voice_text):
+    """Match voice text to 1-indexed option number."""
+    voice_lower = voice_text.lower().strip()
+
+    # Direct number
+    if voice_lower.isdigit():
+        num = int(voice_lower)
+        if 1 <= num <= len(OPTION_LABELS):
+            return num
+        return 1
+
+    # Number words
+    if voice_lower in NUMBER_WORDS:
+        num = NUMBER_WORDS[voice_lower]
+        if 1 <= num <= len(OPTION_LABELS):
+            return num
+        return 1
+
+    # Exact match (case-insensitive)
+    for i, label in enumerate(OPTION_LABELS, 1):
+        if voice_lower == label.lower():
+            return i
+
+    # Partial matches
+    for i, label in enumerate(OPTION_LABELS, 1):
+        if label.lower() in voice_lower or voice_lower in label.lower():
+            return i
+
+    return 1  # Default to first option
+
+def navigate_to_option(option_num):
+    """Navigate using arrow keys: Down arrow (N-1) times, then Enter."""
     time.sleep(0.1)
-    subprocess.run(['pbcopy'], input=text.encode(), check=True)
-    script = """
-    tell application "Terminal"
-        activate
-        delay 0.15
-        tell application "System Events"
-            keystroke "v" using command down
-            delay 0.1
-            key code 36
-        end tell
-    end tell
-    """
-    subprocess.run(['osascript', '-e', script], capture_output=True)
+
+    # Press Down arrow (option_num - 1) times
+    for _ in range(option_num - 1):
+        subprocess.run(['osascript', '-e', 'tell application "System Events" to key code 125'], capture_output=True)
+        time.sleep(0.05)
+
+    # Press Enter to select
+    subprocess.run(['osascript', '-e', 'tell application "System Events" to key code 36'], capture_output=True)
 
 # Poll for response
 start = time.time()
@@ -309,12 +492,14 @@ while time.time() - start < TIMEOUT:
                 label = selected[0]
                 # Handle custom/voice input (Watch sends "__custom__" with message)
                 if label == '__custom__' and message:
-                    inject(message)
+                    # Map voice text to option and navigate
+                    option_num = match_voice_to_option(message)
+                    navigate_to_option(option_num)
                 else:
-                    # Find index of selected label
+                    # Find index of selected label and navigate
                     for idx, opt_label in enumerate(OPTION_LABELS, 1):
                         if opt_label == label:
-                            inject(str(idx))
+                            navigate_to_option(idx)
                             break
         break
     time.sleep(1.5)
@@ -568,6 +753,12 @@ def send_notification(
 ) -> int:
     """Send notification via watchcode CLI (uses relay server)."""
 
+    # DEBUG: Log the notification_id being sent
+    # This is crucial for tracing response matching issues
+    if notification_id:
+        log_debug(f"[Send Notification] notification_id={notification_id}")
+        log_debug(f"[Send Notification] event={event}, requires_action={requires_action}")
+
     # Build command using watchcode CLI
     cmd = ['watchcode', 'notify', '--event', event]
 
@@ -895,6 +1086,11 @@ def check_relay_response(notification_id: str) -> dict | None:
         log_debug("No notification_id for relay polling")
         return None
 
+    # DEBUG: Log the exact notification_id being polled
+    # This helps trace mismatches between Mac and Watch
+    log_debug(f"[Relay Poll] notification_id={notification_id}")
+    log_debug(f"[Relay Poll] auth_token={auth_token[:8]}...")
+
     url = f"{RELAY_URL}/response/{auth_token}/{notification_id}"
 
     try:
@@ -907,18 +1103,26 @@ def check_relay_response(notification_id: str) -> dict | None:
         with urllib.request.urlopen(req, timeout=5) as resp:
             if resp.status == 200:
                 data = json.loads(resp.read().decode())
+                log_debug(f"[Relay Poll] Response: found={data.get('found')}")
                 # Response structure: { found: true, response: { action, timestamp, selected_options, ... } }
                 if data.get('found'):
                     response_data = data.get('response', {})
                     action = response_data.get('action')
                     if action:
                         selected_opts = response_data.get('selected_options', [])
+                        stored_notif_id = response_data.get('notification_id', 'unknown')
                         log_debug(f"✓ Relay response found: action={action}, selected={selected_opts}")
+                        log_debug(f"  stored_notification_id={stored_notif_id}")
                         # Return full response data including selected_options
                         return response_data
+                else:
+                    # Response not found - log for debugging
+                    log_debug(f"[Relay Poll] No response stored for this notification_id")
     except urllib.error.HTTPError as e:
         if e.code != 404:  # 404 just means no response yet
             log_debug(f"Relay HTTP error: {e.code}")
+        else:
+            log_debug(f"[Relay Poll] 404 - endpoint not found or no response")
     except Exception as e:
         log_debug(f"Relay poll error: {e}")
 
@@ -1390,24 +1594,33 @@ def handle_ask_user_question(tool_input: dict, session_id: str, project_name: st
             if selected_label == '__custom__':
                 custom_text = response.get('message', '')
                 if custom_text:
-                    log_debug(f"Custom text from Watch: '{custom_text}'")
-                    # Use inject_keystroke (detached subprocess) for reliable paste
-                    inject_keystroke(custom_text)
+                    log_debug(f"Voice input from Watch: '{custom_text}'")
+                    # Use arrow key navigation for voice input
+                    # This maps voice text to option and navigates with arrow keys
+                    if inject_voice_response(custom_text, options):
+                        log_debug(f"✓ Voice input successfully navigated to matching option")
+                    else:
+                        log_debug(f"⚠️ Voice navigation failed, falling back to terminal")
+                        send_invalidation(notification_id, 'voice_navigation_failed')
                 else:
                     log_debug(f"⚠️ __custom__ selected but no message text received")
                     # Send invalidation - custom input failed
                     send_invalidation(notification_id, 'custom_input_failed')
             else:
-                # Map predefined option to its 1-based index
+                # Predefined option tapped - use arrow navigation instead of typing number
                 option_number = None
                 for idx, opt in enumerate(options, 1):
                     if opt.get('label') == selected_label:
-                        option_number = str(idx)
+                        option_number = idx
                         break
 
                 if option_number:
-                    log_debug(f"Mapped '{selected_label}' → option {option_number}")
-                    inject_keystroke(option_number)
+                    log_debug(f"Mapped '{selected_label}' → option {option_number}, using arrow navigation")
+                    if navigate_to_option(option_number):
+                        log_debug(f"✓ Arrow navigation selected option {option_number}")
+                    else:
+                        log_debug(f"⚠️ Arrow navigation failed for option {option_number}")
+                        send_invalidation(notification_id, 'navigation_failed')
                 else:
                     log_debug(f"⚠️ Could not map '{selected_label}' to option number")
                     # Send invalidation - mapping failed
